/*
   Program Ŀ
    Application: Laserjet library                                           
               :                                                            
    Description: HP LaserJet / PCL 5 print driver                           
               :                                                            
      File Name: LJPRTIV.PRG                                                
               :                                                            
  Req Hardware.: HP LaserJet or fully PCL 5 compatible                      
               : Laser Printer -                                            
                                                                            
  Req Software.: Clipper 5.2e                                               
                                                                            
  Compilation..: Clipper LJPRTIII /A /M /N /P /W                            
               :                                                            
               :                                                            
               :                                                            
  Ĵ
         Author: Kevin Loucks                                               
       Designer: Kevin Loucks          Docs By: Kevin Loucks                
               : Capitol City Software, Inc.                                
               : Box 15303                                                  
               : Minneapolis  MN  55415-0303                                
               :                                                            
               : (612) 376-0054                                             
               :                                                            
               : CI$ 76276,1333                                             
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 08:12:11pm            Time updated: 08:12:11pm            
  Ĵ
      Make File:                                                            
      Exec File:                                                            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved
               : This is an original work created by Kevin Loucks and       
               : placed in the Public Domain By Author 2/29/96              
               : It is provided as is with no warrantees, guarantees        
               : or claims whatsoever. Use at your own risk.                
  
*/


   #include "SET.CH"

   #include "INKEY.CH"

   #include "FILEIO.CH"

   #include "LJPRTIV.CH"

   #include "COMMON.CH"


   // shouldnt need this - but just in case
   memvar getlist

   // absolutely do not include "simpleio.ch" it hoses things real good


   Static aJobContrl := {;
          { "PRINTER_RESET    ", .f. } ,;
          { "NUMBER OF_COPIES ", 1   } ,;
          { "LEFT_OFFSET      ", 0   } ,;
          { "TOP_OFFSET       ", 0   } ,;
          { "SIMPLEX_DUPLEX   ", 0   } ,;
          { "DUPLEX_PAGE_SIDE ", 0   } ,;
          { "JOB_SEPARATION   ", .f. }  ;
          }


   Static aPageCntrl := {;
          {"PAGE_SIZE          ",  2   } ,;
          {"PAGE_LENGTH        ", 66   } ,;
          {"PAPER_SOURCE       ",  1   } ,;
          {"LOGICAL_PAGE_ORIENT",  0   } ,;
          {"PRINT_DIRECTION    ",  0   } ,;
          {"LEFT_MARGIN        ",  0   } ,;
          {"RIGHT_MARGIN       ",  0   } ,;
          {"TOP_MARGIN         ",  0   } ,;
          {"TEXT_LENGTH        ", 60   } ,;
          {"PERFORATION_REGION ",  1   } ,;
          {"HMI                ", 12   } ,;
          {"VMI                ",  8   } ,;
          {"LINE_SPACING       ",  6   } ,;
          {"CLEAR_MARGINS      ", .f.  }  ;
          }

   Static aLineCtrl := {;
          {"CPI      ", 10 } ,;
          {"LPI      ",  6 }  ;
          }

   // font related arrays
   Static aSymbolSet := { ;
       {"ISO 60: Norwegian 1         ", "0D"     } , ;
       {"ISO 4: United Kingdom       ", "1E"     } , ;
       {"ISO 69: French              ", "1F"     } , ;
       {"ISO 21: German              ", "1G"     } , ;
       {"ISO 15: Italian             ", "0I"     } , ;
       {"Microsoft Publishing        ", "6J"     } , ;
       {"DeskTop                     ", "7J"     } , ;
       {"PS Text                     ", "10J"    } , ;
       {"Ventura International       ", "13J"    } , ;
       {"Ventura US                  ", "14J"    } , ;
       {"Ventura ITC Zapf Dingbats   ", "9L"     } , ;
       {"PS ITC Zapf Dingbats        ", "10L"    } , ;
       {"ITC Zapf Dingbats Series 100", "11L"    } , ;
       {"ITC Zapf Dingbats Series 200", "12L"    } , ;
       {"ITC Zapf Dingbats Series 300", "13L"    } , ;
       {"PS Math                     ", "5M"     } , ;
       {"Ventura Math                ", "6M"     } , ;
       {"Math-8                      ", "8M"     } , ;
       {"ECMA-94 Latin 1             ", "0N"     } , ;
       {"ISO 11: Swedish             ", "0S"     } , ;
       {"ISO 17: Spanish             ", "2S"     } , ;
       {"ISO 6: ASCII                ", "0U"     } , ;
       {"Legal                       ", "1U"     } , ;
       {"Roman-8                     ", "8U"     } , ;
       {"Windows                     ", "9U"     } , ;
       {"PC-8                        ", "10U"    } , ;
       {"PC-8 D/N                    ", "11U"    } , ;
       {"PC 850                      ", "12U"    } , ;
       {"Pi Font                     ", "15U"    }   ;
       }

   Static aStyle := {;
       {"Upright (Solid)             ", "0S"    } , ;
       {"Italic                      ", "1S"    } , ;
       {"Condensed                   ", "4S"    } , ;
       {"Condensed Italic            ", "5S"    } , ;
       {"Compressed (Extra Condensed)", "8S"    } , ;
       {"Expanded                    ", "24S"   } , ;
       {"Outline                     ", "32S"   } , ;
       {"Inline                      ", "64S"   } , ;
       {"Shadowed                    ", "128S"  } , ;
       {"Outline Shadowed            ", "160S"  }   ;
       }

   Static aStrokeWt := {;
       {"Ultra Thin           ",        "-7B"   } , ;
       {"Extra Thin           ",        "-6B"   } , ;
       {"Thin                 ",        "-5B"   } , ;
       {"Extra Light          ",        "-4B"   } , ;
       {"Light                ",        "-3B"   } , ;
       {"Demi Light           ",        "-2B"   } , ;
       {"Semi Light           ",        "-1B"   } , ;
       {"Medium (book or text)",        "0B"    } , ;
       {"Semi Bold            ",        "1B"    } , ;
       {"Demi Bold            ",        "2B"    } , ;
       {"Bold                 ",        "3B"    } , ;
       {"Extra Bold           ",        "4B"    } , ;
       {"Black                ",        "5B"    } , ;
       {"Extra Black          ",        "6B"    } , ;
       {"Ultra Black          ",        "7B"    }   ;
       }

   Static aTypeFamil := {;
       {"Courier              ",        "3T"    } , ;
       {"LinePrinter          ",        "0T"    } , ;
       {"Univers              ",        "4148T" } , ;
       {"CG Times             ",        "4101T" } , ;
       {"Antique Olive        ",        "4168T" } , ;
       {"ITC Avant Garde      ",        "4127T" } , ;
       {"Baskerville          ",        "4135T" } , ;
       {"ITC Benguiat         ",        "4158T" } , ;
       {"CG Bodoni            ",        "4149T" } , ;
       {"ITC Bookman          ",        "4143T" } , ;
       {"Brush                ",        "4128T" } , ;
       {"CG Century Schoolbook",        "4119T" } , ;
       {"ITC Cheltenham       ",        "4159T" } , ;
       {"ITC Clearface        ",        "4135T" } , ;
       {"Cooper               ",        "4142T" } , ;
       {"Dom Casual           ",        "4157T" } , ;
       {"ITC Eras             ",        "4172T" } , ;
       {"Futura II            ",        "4110T" } , ;
       {"ITC Galliard         ",        "4126T" } , ;
       {"Garamond             ",        "4197T" } , ;
       {"Gill Sans            ",        "4147T" } , ;
       {"CG Gaudy Old Style   ",        "4138T" } , ;
       {"ITC Korina           ",        "4123T" } , ;
       {"ITC Lubalin Graph    ",        "4174T" } , ;
       {"CG Melliza           ",        "4151T" } , ;
       {"Microstyle           ",        "4175T" } , ;
       {"CG Omega             ",        "4113T" } , ;
       {"CG Palacio           ",        "4111T" } , ;
       {"Park Avenue          ",        "4155T" } , ;
       {"Revue                ",        "4193T" } , ;
       {"Rockwell             ",        "4150T" } , ;
       {"ITC Souvenir         ",        "4112T" } , ;
       {"ITC Tiffany          ",        "4152T" } , ;
       {"CG Triumvirate       ",        "4100T" } , ;
       {"Uncial               ",        "4169T" }   ;
       }

/*
   Function Ŀ
           Name: LJ_SetJob()                                                
               :                                                            
    Description: processes Job Control Commands and                         
               : transmits them to the printer                              
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:21:17pm            Time updated: 07:21:17pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nCommand                                                   
               : nValue                                                     
               : nMethod                                                    
   Return Value: xResult - previous value for the nCommand selected         
  
*/
FUNCTION LJ_SetJob(nCommand, nValue, nMethod )

   local xResult    := ""    ,;
         cCmdPrefix := ESC + "&l"

   default nCommand to PRINTER_RESET
   default nValue   to 0
   default nMethod  to INCHES

   xResult := aJobContrl[nCommand, 2]

   do case
   case nCommand == PRINTER_RESET

      aJobContrl[nCommand, 2] := TRUE
      LJ_Send_CC( ESC + "E" )      // Reset
      setprc( 0, 0)                // set CLIPPER printer Row/Col position

   case nCommand == NUMBER_OF_COPIES
      aJobContrl[nCommand, 2] := nValue
      LJ_Send_CC( cCmdPrefix + str( nValue,1,0) +"X" )

   case nCommand == LEFT_OFFSET
      aJobContrl[nCommand, 2] := nValue
      LJ_Send_CC( cCmdPrefix + LJ_DeciPts( nValue,nMethod ) +"U" )

   case nCommand == TOP_OFFSET
      aJobContrl[nCommand, 2] := nValue
      LJ_Send_CC( cCmdPrefix + LJ_DeciPts( nValue,nMethod )  +"Z" )

   case nCommand == SIMPLEX_DUPLEX
      aJobContrl[nCommand, 2] := nValue
      LJ_Send_CC( cCmdPrefix + str( nValue,1,0) +"S" )

   case nCommand == DUPLEX_PAGE_SIDE
      aJobContrl[nCommand, 2] := nValue
      LJ_Send_CC( ESC + "&a" + str( nValue,1,0) +"G" )

   case nCommand == JOB_SEPARATION
      aJobContrl[nCommand, 2] := TRUE
      LJ_Send_CC( ESC + "&l1T"  )

   endcase

return ( xResult )

/*
   Function Ŀ
           Name: LJ_SetPage()                                               
               :                                                            
    Description: processes Page Control Commands and                        
               : transmits them to the printer                              
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:21:03pm            Time updated: 07:21:03pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nCommand                                                   
               : nValue                                                     
   Return Value: xResult - the previous status for nOption                  
  
*/
Function LJ_SetPage(nCommand, nValue )

   local xResult    := ""

   default nCommand to PAGE_SIZE
   default nValue  to LETTER

   xResult := aPageCntrl[nCommand, 2]
   do case
   case nCommand == PAGE_SIZE
      /*
      page size selection
      : - correct paper tray must be installed for paper size

      Parameters: nValue
                :  1 EXECUTIVE        7.25 x 10.5
                :  2 LETTER           8.5  x 11
                :  3 LEGAL            8.5  x 14
                : 26 A4               210mm x 297mm
                : 80 MONARCH          commercial envelope (3 7/8 x 7 1/2)
                : 81 COMMERCIAL_10    commercial envelope (4 1/8 x 9 1/2)
                : 90 INTERNATIONAL_DL 110mm x 220mm
                : 91 INTERNATIONAL_C5 162mm x 229mm
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + ltrim( str( nValue,2,0 ) ) + "A" )

   case nCommand == PAGE_LENGTH
      /*

      Parameters: nValue
                : 63 EXECUTIVE_6_LPI_PORT
                : 84 EXECUTIVE_8_LPI_PORT
                : 43 EXECUTIVE_6_LPI_LAND
                : 58 EXECUTIVE_8_LPI_LAND
                : 66 LETTER_6_LPI_PORT
                : 88 LETTER_8_LPI_PORT
                : 51 LETTER_6_LPI_LAND
                : 68 LETTER_8_LPI_LAND
                : 84 LEGAL_6_LPI_PORT
                :112 LEGAL_8_LPI_PORT
                :  0 LEGAL_6_LPI_LAND
                :  0 LEGAL_8_LPI_LAND
                : 70 A4_6_LPI_PORT
                : 93 A4_8_LPI_PORT
                : 49 A4_6_LPI_LAND
                : 66 A4_8_LPI_LAND
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + ltrim( str( nValue,3,0 ) ) + "P" )


   case nCommand == PAPER_SOURCE
      /*
       Purpose...:  select source for paper

       Parameters: nValue
                 : 0 = eject page
                 : 1 = upper tray        (default)
                 : 2 = manual input
                 : 3 = manual envelope
                 : 4 = lower tray
                 : 6 = envelope feeder
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + str( nValue,1,0) + "H"        )

   case nCommand == LOGICAL_PAGE_ORIENT
      /*
      Purpose...:  select page orientation

      Parameters: nValue
                  0 = PORTRAIT
                  1 = LANDSCAPE
                  2 = reverse PORTRAIT
                  3 = reverse LANDSCAPE
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + str( nValue,1,0) + "O"          )

   case nCommand == PRINT_DIRECTION
      /*
      Purpose...:  select direction of print

      Parameters: nValue
                  0 = PORTRAIT
                 90 = LANDSCAPE
                180 = reverse PORTRAIT
                270 = reverse LANDSCAPE
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&a" + ltrim( str( nValue,3,0 ) ) + "P" )

   case nCommand == LEFT_MARGIN
      /*
      Purpose...:  set the left margin position

      Parameters: nValue = column # 0 - n
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + ltrim( str( nValue,3,0 ) ) + "L" )

   case nCommand == RIGHT_MARGIN
      /*
      Purpose...:  set the right margin position

      Parameters: nValue = column # 0 - n
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + ltrim( str( nValue,3,0 ) ) + "M" )

   case nCommand == TOP_MARGIN
      /*
      Purpose...:  set the top margin position

      Parameters: nValue = Row # 0 - n
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + ltrim( str( nValue,3,0 ) ) + "E" )

   case nCommand == TEXT_LENGTH
      /*
      Purpose...:  set the text length
                   (effectively the bottom margin) position

      Parameters: nValue = # of lines
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + ltrim( str( nValue,3,0 ) ) + "F" )

   case nCommand == PERFORATION_REGION
      /*
      Purpose...:  enable / disable perforation region

      Parameters: nValue
                : 0 = disable
                : 1 = enable
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + str( nValue,1,0) + "L"        )

   case nCommand == HMI
      /*
      Purpose...:  set horizontal motion indicator

      Parameters: nValue = # of 1/120th of an inch increments
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&k" + ltrim( str( nValue,5,0 ) ) + "H" )

   case nCommand == VMI
      /*
      Purpose...:  set vertical motion indicator

      Parameters: nValue = # of 1/48th of an inch increments
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + ltrim( str( nValue,5,0 ) ) + "C" )


   case nCommand == LINE_SPACING

      /*
      Purpose...:  set line spacing # of Lines per Inch

      Parameters: nValue =
                :  1    LPI_1
                :  2    LPI_2
                :  3    LPI_3
                :  4    LPI_4
                :  6    LPI_6
                :  8    LPI_8
                : 12    LPI_12
                : 16    LPI_16
                : 24    LPI_24
                : 48    LPI_48
      */

      aPageCntrl[nCommand, 2]  := nValue
      aLineCtrl[2, 2]          := nValue
      LJ_Send_CC( ESC + "&l" + ltrim( str( nValue,2,0 ) ) + "D" )

   case nCommand == CLEAR_MARGINS

      /* Parameters: nValue =
                   :  9    reset left & right margins
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "9"  )

   case nCommand == PAPER_DESTINATION
      /*
       Purpose...:  select destination for paper

       Parameters: nValue
                 : 1 = upper output bin        (default)
                 : 2 = lower output bin
      */

      aPageCntrl[nCommand, 2]  := nValue
      LJ_Send_CC( ESC + "&l" + str( nValue,1,0 ) + "G" )

   endcase

return (xResult )


/*
   Function Ŀ
           Name: LJ_PosRC()                                                 
               :                                                            
    Description: positions the print cursor via row                         
               : and col coordinates does not use                           
               : cursor push & pop                                          
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:20:38pm            Time updated: 07:20:38pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nRow                                                       
               : nCol                                                       
               : cRelRow                                                    
               : cRelCol                                                    
   Return Value: nil                                                        
  
*/
Function LJ_PosRC(nRow, nCol, cRelRow, cRelCol )

   local cS := ""  ,;
         cY := trim( ltrim( str(nRow,3,0) ) ) + "r",;
         cX := trim( ltrim( str(nCol,3,0) ) ) + "C"

   default cRelRow to ""
   default cRelCol to ""

   cS := ESC + "&a" + if(cRelRow == "","",cRelRow ) + cY + ;
                      if(cRelCol == "","",cRelCol ) + cX

   LJ_Send_NC( cS )

return nil

/*
   Function Ŀ
           Name: LJ_PosDeci()                                               
               :                                                            
    Description: positions the print cursor via inches                      
               : converted to decipoints does not use                       
               : cursor push & pop                                          
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:20:25pm            Time updated: 07:20:25pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nInRow                                                     
               : nInCol                                                     
               : cRelRow                                                    
               : cRelCol                                                    
   Return Value: nil                                                        
  
*/
Function LJ_PosDeci(nInRow,nInCol, cRelRow, cRelCol )

   local cS := "" ,;
         cY := "" ,;
         cX := ""

   default cRelRow to ""
   default cRelCol to ""

   cY := trim( ltrim( str( RoundIt(nInRow * 720,2 ), 8,2) ) ) + "v"
   cX := trim( ltrim( str( RoundIt(nInCol * 720,2 ), 8,2) ) ) + "H"

   cS := ESC + "&a" + if(cRelRow == "","",cRelRow ) + cY  +;
                      if(cRelCol == "","",cRelCol ) + cX

   LJ_Send_NC( cS )

return nil


/*
   Function Ŀ
           Name: LJ_PosDots()                                               
               :                                                            
    Description: positions the print cursor via inches converted            
               : to decipoints does not use cursor push & pop               
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:20:17pm            Time updated: 07:20:17pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nInRow                                                     
               : nInCol                                                     
               : cRelRow                                                    
               : cRelCol                                                    
   Return Value: nil                                                        
  
*/
Function LJ_PosDots(nInRow, nInCol, cRelRow, cRelCol )

   local cS := "" ,;
         cY := "" ,;
         cX := ""

   default cRelRow to ""
   default cRelCol to ""

   cY := trim( ltrim( str( RoundIt(nInRow * 300,2 ), 8,2) ) ) + "y"
   cX := trim( ltrim( str( RoundIt(nInCol * 300,2 ), 8,2) ) ) + "X"

   cS := ESC + "&a" + if(cRelRow == "","",cRelRow )+ cY +;
                      if(cRelCol == "","",cRelCol )+ cX

   LJ_Send_NC( cS )

return nil



/*
   Function Ŀ
           Name: LJ_HalfLn()                                                
               :                                                            
    Description: performs  a half line feed                                 
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:20:07pm            Time updated: 07:20:07pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: None                                                       
   Return Value: nil                                                        
  
*/
Function LJ_HalfLn()

   local cS := ESC + "="

   LJ_Send_NC( cS )

return nil

/*
   Function Ŀ
           Name: LJ_SetLn()                                                 
               :                                                            
    Description: sets line control                                          
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:19:59pm            Time updated: 07:19:59pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nOption                                                    
               : xNewvalue                                                  
   Return Value: xOldValue                                                  
  
*/
Function LJ_SetLn( nOption, xNewvalue )

   local xOldValue

   xOldValue := aLineCtrl[ nOption, 2 ]
   aLineCtrl[ nOption,2 ] := xNewValue

return ( xOldValue )



/*
   Function Ŀ
           Name: LJ_LnTerm()                                                
               :                                                            
    Description: changes the line termination behavior                      
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:19:52pm            Time updated: 07:19:52pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nOption                                                    
               :  DEFAULT                  0                                
               :  CR_CRLF                  1                                
               :  LF_CRLF_CRFF             2                                
               :  CR_CRLF_CRLF_CRFF        3                                
   Return Value: nil                                                        
  
*/
Function LJ_LnTerm(nOption)

   local cS := ESC + "&k" + str(nOption,1,0) + "G"

   LJ_Send_NC( cS )

return nil

/*
   Function Ŀ
           Name: LJ_PushPop()                                               
               :                                                            
    Description: pushes or pops printer cursor position on or off stack     
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:19:30pm            Time updated: 07:19:30pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nOption                                                    
               :   PUSH    0                                                
               :   POP     1                                                
   Return Value: nil                                                        
  
*/
Function LJ_PushPop( nOption )

   local lConsole, ;
         cDevice,  ;
         lPrinter, ;
         nRow,     ;
         nCol,     ;
         cS := ESC + "&f" + str(nOption,1,0) + "S"

   lConsole := set(_SET_CONSOLE, .F.)        // SET CONSOLE OFF
   cDevice  := set(_SET_DEVICE , "PRINTER")  // SET DEVICE TO PRINTER
   lPrinter := set(_SET_PRINTER, .T.)        // SET PRINTER ON

   nRow     := prow()              // save CLIPPER printer Row position
   nCol     := pcol()              // save CLIPPER printer Col position

   qqout( cS )

   setprc(nRow, nCol)              // restore CLIPPER printer Row/Col position

   set(_SET_DEVICE,  cDevice )     // Restore device  setting
   set(_SET_PRINTER, lPrinter)     // Restore printer setting
   set(_SET_CONSOLE, lConsole)     // Restore console setting

return nil

/*
   Function Ŀ
           Name: LJ_Say_YX()                                                
               :                                                            
    Description: Position a text string at an absolute                      
               : position on the page                                       
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:18:54pm            Time updated: 07:18:54pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nXYfmt   = describes how to evaluate incomming parametest  
               :            INCHES   DOTS    ROWCOL  MILLIMETER             
               : nY       = column (X) dot coordinates in string format     
               : nX       = row (Y) dot coordinates in string format        
               : xVariable= variable/data    to pass to printer             
               : cPicture = character string to picture for variable format 
   Return Value: nil                                                        
  
*/
function LJ_Say_YX(nXYfmt, nY, nX, xVariable, cPicture )

   local cPrefix, ;
         cX,      ;
         cY

   do case
      case nXYfmt == ROWCOL        //  PArms passed in Row/Col config
           cY := lj_dots( nY,ROW_POSITION)
           cX := lj_dots( nX,COL_POSITION)

      case nXYfmt == INCHES        //  parms  passed in INCH config
           cY := lj_dots( nY, INCHES )
           cX := lj_dots( nX, INCHES )

      case nXYfmt == DOTS          //  parms  passed in numeric dot config
           cY := lj_dots( nY,DOTS)
           cX := lj_dots( nX,DOTS)

      case nXYfmt == MILLIMETER    //  parms  passed in  MILLIMETER config
           cY := lj_dots( nY, MILLIMETER )
           cX := lj_dots( nX, MILLIMETER )
   endcase

   cPrefix := ESC + "*p" + cX + "x" + cY + "Y"

   if cPicture <> nil
      LJ_Send_CC( cPrefix + transform(xVariable, cPicture ) )
   else
      LJ_Send_CC( cPrefix + xVariable )
   endif

return nil

/*
   Function Ŀ
           Name: LJ_Say_YXF()                                               
               :                                                            
    Description: Position a text string at an absolute position                                                           
               : on the page and format for print to user                                                                 
               : defined parameters                                                                                       
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 08:00:47pm            Time updated: 08:00:47pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nXYfmt    = describes how to evaluate incomming parameters 
               : nY        = row (Y) dot coordinates in string format       
               : nX        = column (X) dot coordinates in string format    
               : xVariable = variable / data  to pass to printer            
               : cFontStart= the font to invoke before printing the string  
               : cFontEnd  = the font to invoke after printing the string   
               : nUnderLn  = indicates whether or not to use nuderlining    
               : cPicture  = character string to picture for variable format
   Return Value: nil                                                        
  
*/
function LJ_Say_YXF( nXYfmt, nY, nX, ;
                     xVariable, cFontStart, cFontEnd, nUnderLn, cPicture )

   local cX,;
         cY,;
         cPosition, ;
         nCurFont,  ;
         cUL_on,;
         cUL_off

   default cFontStart to ""
   default cFontEnd   to ""
   default nUnderLn   to UNDERLINE_OFF


   do case
      case nXYfmt == ROWCOL
           //  parms passed in Row/Col config
           cY := lj_dots( nY,ROW_POSITION )
           cX := lj_dots( nX,COL_POSITION )

      case nXYfmt == INCHES
           //  parms  passed in INCH config
           cY := lj_dots( nY,INCHES )
           cX := lj_dots( nX,INCHES )

      case nXYfmt == DOTS
           //  parms  passed in numeric dot config
           cY := lj_dots( nY,DOTS )
           cX := lj_dots( nX,DOTS )

      case nXYfmt == MILLIMETER
           //  parms passed in millimeter config
           cY := lj_dots( nY,MILLIMETER )
           cX := lj_dots( nX,MILLIMETER )
   endcase

   cPosition := ESC + "*p" + cX + "x" + cY + "Y"

   do case
   case nUnderLn == FLOATING_UNDERLINE_ON
      cUL_on    := ESC + "&d"  + str( nUnderLn, 1,0) + "D"
      cUL_off   := ESC + "&d@"
   case nUnderLn == FIXED_UNDERLINE_ON
      cUL_on    := ESC + "&d"  + str( nUnderLn, 1,0) + "D"
      cUL_off   := ESC + "&d@"
   otherwise
      cUL_on    := ""
      cUL_off   := ""
   endcase

   if cPicture <> nil
      LJ_Send_CC( cPosition + cFontStart + cUL_on + ;
                  transform( xVariable, cPicture) + cUL_off + cFontEnd )
   else
      LJ_Send_CC( cPosition + cFontStart + cUL_on + ;
                  xVariable + cUL_off + cFontEnd )
   endif

return nil

/*
   Function Ŀ
           Name: LJ_Send_CC()                                               
               :                                                            
    Description: transmit control codes and string information              
               : to the printer WITHOUT CHANGING  the row and               
               : column OR Cursor settings                                  
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:17:33pm            Time updated: 07:17:33pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: cString = a character string containing                    
               :           whatever you want to send to the printer         
   Return Value: nil                                                        
  
*/
function LJ_Send_CC( cString )

   local lConsole, ;
         cDevice,  ;
         lPrinter, ;
         nRow,     ;
         nCol

   lConsole := set(_SET_CONSOLE, .F.)        // SET CONSOLE OFF
   cDevice  := set(_SET_DEVICE , "PRINTER")  // SET DEVICE TO PRINTER
   lPrinter := set(_SET_PRINTER, .T.)        // SET PRINTER ON

   nRow     := prow()              // save CLIPPER printer Row position
   nCol     := pcol()              // save CLIPPER printer Col position

   QQout( ESC + "&f0S" )  // save internal printer cursor position
   QQout( cString )       // print whatever it is
   QQout( ESC + "&f1S" )  // restore internal printer cursor position

   setprc(nRow, nCol)              // restore CLIPPER printer Row/Col position

   set(_SET_DEVICE,  cDevice )     // Restore device  setting
   set(_SET_PRINTER, lPrinter)     // Restore printer setting
   set(_SET_CONSOLE, lConsole)     // Restore console setting

return nil


/*
   Function Ŀ
           Name: LJ_Send_NC()                                               
               :                                                            
    Description: transmit control codes and string information              
               : to the printer CHANGING the row and column and             
               : Cursor settings                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:17:27pm            Time updated: 07:17:27pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: cString = a character string containing whatever           
               :           you want to send to the printer                  
   Return Value: nil                                                        
  
*/
function LJ_Send_NC( cString )

   local lConsole,;
         cDevice, ;
         lPrinter,;
         nRow,    ;
         nCol

   lConsole := set(_SET_CONSOLE, .F.)        // SET CONSOLE OFF
   cDevice  := set(_SET_DEVICE , "PRINTER")  // SET DEVICE TO PRINTER
   lPrinter := set(_SET_PRINTER, .T.)        // SET PRINTER ON

   nRow     := prow()              // save CLIPPER printer Row position
   nCol     := pcol()              // save CLIPPER printer Col position

   qqout( cString )

   setprc(nRow, nCol)              // restore CLIPPER printer Row/Col position

   set(_SET_DEVICE,  cDevice )     // Restore device  setting
   set(_SET_PRINTER, lPrinter)     // Restore printer setting
   set(_SET_CONSOLE, lConsole)     // Restore console setting

return nil

/*
   Function Ŀ
           Name: LJ_Dots()                                                  
               :                                                            
    Description: convert character units to dots for absolute printer       
               : positioning                                                
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:17:14pm            Time updated: 07:17:14pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nUnits  = number of char units to be converted to dots     
               : nMethod = Method of evaluating the units                   
               :           COL = columns   ROW = rows                       
               :           IN  = inches    DT  = dots                       
               :                                                            
   Return Value: cResult - character string of number of dots               
  
*/
static function LJ_Dots( nUnits, nMethod)

   local cResult    := ""

   do case
      case nMethod == COL_POSITION
         cResult := alltrim( STR( nUnits * ;
                      RoundIt( 300 / aLineCtrl[1,2],4),10, 0) )

      case nMethod == ROW_POSITION
         cResult := alltrim( STR( nUnits * ;
                     RoundIt(300 / aLineCtrl[2,2],4),10, 0) )

      case nMethod == INCHES             // inch = 300 dots
         cResult := alltrim( STR( nUnits * 300, 10, 0) )

      case nMethod == MILLIMETER         // millimeter = 11.811 dots
         cResult := alltrim( STR( RoundIt(nUnits * 11.811,4), 10, 0) )

      case nMethod == DOTS
         cResult := alltrim( STR( nUnits, 10, 0) )
   endcase

RETURN (cResult)

/*
   Function Ŀ
           Name: LJ_Dots600()                                               
               :                                                            
    Description: convert character units to dots for absolute printer       
               : positioning                                                
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:17:07pm            Time updated: 07:17:07pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nUnits  = number of char units to be converted to dots     
               : nMethod = Method of evaluating the units                   
               :           COL = columns   ROW = rows                       
               :           IN  = inches    DT  = dots                       
   Return Value: cResult - character string of number of dots               
  
*/
static function LJ_Dots600( nUnits, nMethod)

   local cResult    := ""

   do case
      case nMethod == COL_POSITION
         cResult := alltrim( STR( nUnits * ;
                      RoundIt( 600 / aLineCtrl[1,2],4),10, 0) )

      case nMethod == ROW_POSITION
         cResult := alltrim( STR( nUnits * ;
                     RoundIt( 600 / aLineCtrl[2,2],4),10, 0) )

      case nMethod == INCHES             // inch = 300 dots
         cResult := alltrim( STR( nUnits * 600, 10, 0) )

      case nMethod == MILLIMETER         // millimeter = 23.622 dots
         cResult := alltrim( STR( RoundIt(nUnits * 23.622,4), 10, 0) )

      case nMethod == DOTS
         cResult := alltrim( STR( nUnits, 10, 0) )
   endcase

RETURN (cResult)

/*
   Function Ŀ
           Name: LJ_DeciPts()                                               
               :                                                            
    Description: convert character units to decipoints for                  
               : absolute print cursor positioning                          
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:16:40pm            Time updated: 07:16:40pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nUnits = number of char units to be converted to dots      
               : nMethod= Method of evaluating the units                    
               :          COL = columns   ROW = rows                        
               :          IN  = inches    DT  = dots                        
   Return Value: cResult - character string of number of dots               
  
*/
static function LJ_DeciPts( nUnits, nMethod)

   local cResult    := ""

   do case
      case nMethod == COL_POSITION
         cResult := alltrim( STR( nUnits * RoundIt(720 / aLineCtrl[1,2],4),10, 0) )

      case nMethod == ROW_POSITION
         cResult := alltrim( STR( nUnits * RoundIt(720 / aLineCtrl[2,2],4),10, 0) )

      case nMethod == INCHES
         cResult := alltrim( STR( nUnits * 720, 10, 0) )

      case nMethod == MILLIMETER
         // millimeter = 28.3465 decipoints per millimeter
         cResult := alltrim( STR( RoundIt(nUnits * 28.3465,4), 10, 0) )

   endcase

RETURN (cResult)


/*
   Function Ŀ
           Name: LJ_Font_ID()                                               
               :                                                            
    Description: select a soft font by its assigned id number               
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:16:33pm            Time updated: 07:16:33pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nFontNo                                                    
   Return Value: nil                                                        
  
*/
FUNCTION LJ_Font_ID( nFontNo  )
   local cS := ""

   default  nFontNo to 1

   cS := ESC + "*c"  + ltrim( str(nFontNo,5,0)) + "D"

   LJ_Send_CC( cS )

return nil

/*
   Function Ŀ
           Name: Lj_FontMng()                                               
               :                                                            
    Description: Manages the manipulation of soft fonts including           
               : the ability to Download a soft font to the printer,        
               : assigns an ID and  define the lifespan of a                
               : font(temporary/permanent)                                  
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:16:21pm            Time updated: 07:16:21pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved
  Ĵ
      Arguments: nFontMngmt - font control options                          
               : DELETE_ALL          0 = delete all soft fonts              
               : DELETE_ALL_TEMP     1 = delete all temporary soft fonts    
               : DELETE_LAST_FONT    2 = delete soft referenced by cFontID  
               : DELETE_CHAR_CODE    3 = delete soft refd by cFontID & char 
               : MAKE_TEMPORARY      4 = make soft font temporary (cFontID) 
               : MAKE_PERMANENT      5 = make soft font permanent (cFontID) 
               : COPY_FONT_AS_TEMP   6 = copy/assign current font temporary 
               : SELECT_FONT_PRIM    7 = select soft font id as primary font
               : SELECT_FONT_SECOND  8 = select soft font as secondary      
               : SELECT_DFLT_PRIM    9 = select default font as primary font
               : SELECT_DFLT_SECOND 10 = select default font as secondary   
               :                                                            
               : nFontOpt - field with numeric font id for printer          
               : cFontOpt - character field with additional info            
   Return Value: lResult  - TRUE or FALSE = success of routine              
  
*/
FUNCTION Lj_FontMng(nFontMngmt, nFontOpt, cFontOpt  )

   local cPrefix := ESC + "*c"
   local cSuffix := ltrim( str( nFontMngmt,2,0) ) +"F"
   local cS      := ""
   local lResult := .t.

   default nFontMngmt to 0
   default nFontOpt   to 333
   default cFontOpt   to ""

   DO CASE
      CASE nFontMngmt == DELETE_ALL
         cS := cPrefix + cSuffix

      CASE nFontMngmt == DELETE_ALL_TEMP
         cS := cPrefix + cSuffix

      CASE nFontMngmt == DELETE_LAST_FONT
         cS := cPrefix + alltrim(str(nFontOpt,5,0))   + "d" + cSuffix

      CASE nFontMngmt == DELETE_CHAR_CODE
         cS := cPrefix + alltrim(str(nFontOpt,5,0))   + "d" + ;
               asc( left(cFontOpt,1) ) + "e" +  cSuffix

      CASE nFontMngmt == MAKE_TEMPORARY
         cS := cPrefix + alltrim(str(nFontOpt,5,0))   + "d" + cSuffix

      CASE nFontMngmt == MAKE_PERMANENT
         cS := cPrefix + alltrim(str(nFontOpt,5,0))   + "d" + cSuffix

      CASE nFontMngmt == COPY_FONT_AS_TEMP
         cS := cPrefix + alltrim(str(nFontOpt,5,0))   + "d" + cSuffix

      CASE nFontMngmt == SELECT_FONT_PRIM
         cS := ESC + "(" + alltrim(str(nFontOpt,5,0)) + "d" + cSuffix

      CASE nFontMngmt == SELECT_FONT_SECOND
         cS := ESC + ")" + alltrim(str(nFontOpt,5,0)) + "d" + cSuffix

      CASE nFontMngmt == SELECT_DFLT_PRIM
         cS := ESC + "(3@"

      CASE nFontMngmt == SELECT_DFLT_SECOND
         cS := ESC + ")3@"
   ENDCASE

   LJ_Send_CC( cS )

return (lResult)


/*
   Function Ŀ
           Name: LJ_Font_SS()                                               
               :                                                            
    Description: builds a font SELECTION STRING and returns it              
               : to the calling function                                    
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:16:08pm            Time updated: 07:16:08pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nPrimSec                                                   
               : nSymSet                                                    
               : nSpacing                                                   
               : nPitch                                                     
               : nHeight                                                    
               : nStyle                                                     
               : nStroke                                                    
               : nTypeFamil                                                 
   Return Value: cFontString - character string of number of dots           
  
*/
function LJ_Font_SS( nPrimSec, nSymSet,;
                     nSpacing, nPitch,  nHeight, ;
                     nStyle,   nStroke, nTypeFamil )

   local cFontString := ""

   Default nPrimSec   to PRIMARY_SYMBOL_SET
   Default nSymSet    to PC_8
   Default nSpacing   to FIXED_SPACING
   Default nPitch     to 10
   Default nHeight    to 12
   Default nStyle     to UPRIGHT_SOLID
   Default nStroke    to MEDIUM_BOOK_OR_TEXT
   Default nTypeFamil to COURIER

   cFontString := ESC + if( nPrimSec == PRIMARY_SYMBOL_SET, "(", ")" ) +;
                        aSymbolSet[nSymSet, 2]

   cFontString := cFontString + ESC +;
                  if( nPrimSec == PRIMARY_SYMBOL_SET  , "(s",")s" ) + ;
                     str(nSpacing,1,0) + "p"

   if nSpacing == FIXED_SPACING
      cFontString := cFontString + str(nPitch,6,2) + "h"
   endif

   cFontString := cFontString + str(nHeight,6,2) + "v" + ;
                  lower( aStyle[ nStyle, 2] )    +       ;
                  lower( aStrokeWt[nStroke, 2] ) +       ;
                  aTypeFamil[ nTypeFamil, 2]


return (cFontString)



/*
   Function Ŀ
           Name: LJ_Font_AS()                                               
               :                                                            
    Description: changes a particular font attribute                        
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:15:57pm            Time updated: 07:15:57pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nOption                                                    
               :    Symbol Set                                              
               :    Spacing                                                 
               :    Pitch                                                   
               :    Height                                                  
               :    Style                                                   
               :    Stroke                                                  
               :    Type Family                                             
               :                                                            
               :                                                            
               : nAttribute                                                 
   Return Value: cString                                                    
  
*/
function LJ_Font_AS( nOption,  nAttribute )

   local cString := ""

   do case
   case nOption == SYMBOL_SET
        cString := ESC + "(" +  aSymbolSet[ nAttribute, 2]

   case nOption == SPACING
        cString := ESC + "(s" + str(nAttribute,1,0) + "P"

   case nOption == PITCH
        cString := ESC + "(s" + ltrim(str(nAttribute,6,2)) + "H"

   case nOption == POINT_SIZE
        cString := ESC + "(s" + ltrim(str(nAttribute,6,2)) + "V"

   case nOption == STYLE
        cString := ESC + "(s" + aStyle[ nAttribute, 2]

   case nOption == STROKE_WEIGHT
        cString := ESC + "(s" + aStrokeWt[ nAttribute, 2]

   case nOption == TYPEFACE_FAMILY
        cString := ESC + "(s" + aTypeFamil[ nAttribute, 2]

   endcase


return (cString)



/*
   Function Ŀ
           Name: LJ_TranDat()                                               
               :                                                            
    Description: transparant print data controls                            
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:15:49pm            Time updated: 07:15:49pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nOption = number of bytes of transparent print data        
               : nChar   = decimal value of data = 0, 7-15, 27              
   Return Value: Nil                                                        
  
*/
FUNCTION LJ_TranDat( nOption,nChar )

   local cS := ""

   default  nOption to 1

   cS := ESC + "&p"  + str( nOption ,1,0) + "X" + ltrim( str(nChar,3,0) )

   LJ_Send_CC( cS )

return Nil


/*
   Function Ŀ
           Name: Lj_MacroID()                                               
               :                                                            
    Description: specify a macro id for use by subsequent macro commands    
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:15:16pm            Time updated: 07:15:16pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nMacroNo =macro id number 1 - 32767                        
   Return Value: nil                                                        
  
*/
FUNCTION Lj_MacroID( nMacroNo )

   local cS := ""

   default  nMacroNo to 1

   cS := ESC + "&f"  + ltrim( str(nMacroNo,5,0)) + "Y"

   LJ_Send_CC( cS )

return nil

/*
   Function Ŀ
           Name: LJ_MacroCt()                                               
               :                                                            
    Description: controls the installation, definition execution            
               : and deletion of macros                                     
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:14:46pm            Time updated: 07:14:46pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nMacroOpt                                                  
               :    START_DEFINITION         0                              
               :    STOP_DEFINITION          1                              
               :    EXECUTE_MACRO            2                              
               :    CALL_MACRO               3                              
               :    ENABLE_AUTO_OVERLAY      4                              
               :    DISABLE_AUTO_OVERLAY     5                              
               :    DELETE_ALL_MACROS        6                              
               :    DELETE_ALL_TEMP_MACROS   7                              
               :    DELETE_MACRO_LAST        8                              
               :    MAKE_MACRO_TEMPORARY     9                              
               :    MAKE_MACRO_PERMANENT    10                              
   Return Value: nil                                                        
  
*/
FUNCTION LJ_MacroCt( nMacroOpt  )

   local cS := ""

   default  nMacroOpt to 6    // clears all macros from memory

   cS := ESC + "&f"  + ltrim( str(nMacroOpt,2,0)) + "X"

   LJ_Send_CC( cS )

return nil


/*
   Function Ŀ
           Name: LJ_RastSet()                                               
               :                                                            
    Description: control raster graphics functions                          
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:14:32pm            Time updated: 07:14:32pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nCommand                                                   
               :    GRAPHICS_RESOLUTION    1                                
               :    PRESENTATION_MODE      2                                
               :    RASTER_HEIGHT          3                                
               :    RASTER_WIDTH           4                                
               :    START_RASTER_GRAPHICS  5                                
               :    END_RASTER_GRAPHICS    6                                
               :    RASTER_Y_OFFSET        7                                
               :    COMPRESSION_MODE       8                                
               :                                                            
               : nOption                                                    
   Return Value: nil                                                        
  
*/
function LJ_RastSet( nCommand, nOption )

   local cS := ""

   do case
   case nCommand == GRAPHICS_RESOLUTION
      cS := ESC + "*t" + ltrim( str( nOption,3,0) ) + "R"

   case nCommand == PRESENTATION_MODE
      cS := ESC + "*r" + str(nOption,1,0) + "F"

   case nCommand == RASTER_HEIGHT
      cS := ESC + "*r" + ltrim( str(nOption,4,0) ) + "T"

   case nCommand == RASTER_WIDTH
      cS := ESC + "*r" + ltrim( str(nOption,4,0) ) + "S"

   case nCommand == START_RASTER_GRAPHICS
      cS := ESC + "*r" + str(nOption,1,0) + "A"

   case nCommand == END_RASTER_GRAPHICS
      cS := ESC + "*rB"

   case nCommand == RASTER_Y_OFFSET
      cS := ESC + "*b" + ltrim( str(nOption,4,0) ) + "Y"

   case nCommand == COMPRESSION_MODE
      cS := ESC + "*b" + str(nOption,1,0) + "M"

   endcase


   lj_send_nc( cS )

return nil


/*
   Function Ŀ
           Name: LJ_RasTran()                                               
               :                                                            
    Description: transfer an array of decimal equivlants of binary bit      
               : patterns                                                   
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:14:21pm            Time updated: 07:14:21pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: aRaster = array of values to print                         
   Return Value: nil                                                        
  
*/
function LJ_RasTran( aRaster )

   local cS     := ""
   local nI     := 0
   local nII    := 0

   local nbytes := 0


   for nI  = 1 to len( aRaster )

      nBytes := len(aRaster [ nI  ] )

      cS     := ESC + "*b"+ ltrim( str(nBytes,5,0) ) + "W"

      for nII  = 1 to nBytes

         cS := cs + chr( aRaster[nI, NII ] )

      next nII

      lj_send_nc( cS )

   next nI


return nil


/*
   Function Ŀ
           Name: LJ_Box()                                                   
               :                                                            
    Description: draw a box on the page where-ever you like                 
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:14:02pm            Time updated: 07:14:02pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved
  Ĵ
      Arguments: aBoxParams = describes how to evaluate incomming array     
               : BOX_PARAM_format                                           
               :     Inches                                                 
               :     Dots                                                   
               :     Row / Column                                           
               :     millimeters                                            
               : BOX_top_line       = top row (Y) of box                    
               : BOX_left_side      = left most column (X) of the box       
               : BOX_bottom_line    = bottom row (Y) of the box             
               : BOX_right_side     = right most column (X) of the box      
               : BOX_line_thickness = thickness of the box line in inches   
               : BOX_RAF_option     = numeric Rectangular Area Fill command 
               : BOX_line_pattern   = pattern for line density              
               : BOX_shadow         = shadow indicator for box / 5 options  
               :     SHADOW_NONE                                            
               :     SHADOW_NE      north east orientation                  
               :     SHADOW_SE      south east orientation                  
               :     SHADOW_NW      north west orientation                  
               :     SHADOW_SW      south west orientation                  
               :                                                            
   Return Value: NIL                                                        
  
*/
Function LJ_Box( aBoxParams )

   local cTopRow,  cLeftCol,  cRightCol,  cBotRow,  cWidth,   cThick, cHeight
   local cSTopRow, cSLeftCol, cSRightCol, cSBotRow, cBotWidth

   do while len( aBoxParams) < 9
      AADD( aBoxParams, nil )
   enddo
   if BOX_PARAM_format == MILLIMETER
      cThick := lj_dots( BOX_line_thickness, MILLIMETER )
   else
      cThick := lj_dots( BOX_line_thickness, INCHES )
   endif


   if BOX_PARAM_format == ROWCOL

      // parameters were passed in Row/Col to dots configuration
      cBotWidth := str( val(lj_dots( ABS(BOX_right_side -  BOX_left_side),;
                                COL_POSITION )) + val( cThick ), 10,0 )

      cwidth    := lj_dots( ABS(BOX_right_side -  BOX_left_side),COL_POSITION )
      cHeight   := lj_dots( ABS(BOX_bottom_line - BOX_top_line), ROW_POSITION )

      cTopRow   := lj_dots( BOX_top_line,     ROW_POSITION )
      cLeftCol  := lj_dots( BOX_left_side,    COL_POSITION )
      cRightCol := lj_dots( BOX_right_side,   COL_POSITION )
      cBotRow   := lj_dots( BOX_bottom_line,  ROW_POSITION )

   Else

      //  parameters were passed in:
      //  INCH to dots,  MILLIMETER to dots  or  numeric DOT config
      cBotWidth := str( val(lj_dots( ABS(BOX_right_side -  BOX_left_side),;
                                BOX_PARAM_format )) + val( cThick ), 10,0 )

      cwidth  := LJ_Dots( ABS(BOX_right_side - BOX_left_side),BOX_PARAM_format )
      cHeight := LJ_Dots( ABS(BOX_bottom_line - BOX_top_line),BOX_PARAM_format )

      cTopRow   := LJ_Dots( BOX_top_line,     BOX_PARAM_format )
      cLeftCol  := LJ_Dots( BOX_left_side,    BOX_PARAM_format )
      cRightCol := LJ_Dots( BOX_right_side,   BOX_PARAM_format )
      cBotRow   := LJ_Dots( BOX_bottom_line,  BOX_PARAM_format )
   endif

   // draw top line first from top left to top right
   LJ_RAF( cTopRow, cLeftCol,  cWidth,  cThick, ;
            BOX_RAF_option,  BOX_line_pattern )

   // left side line from left top of box to left bottom
   LJ_RAF( cTopRow, cLeftCol,  cThick, cHeight, ;
            BOX_RAF_option,  BOX_line_pattern )

   // Right side line from right top of box to right bottom
   LJ_RAF( cTopRow, cRightCol, cThick, cHeight, ;
            BOX_RAF_option,  BOX_line_pattern )

   // draw bottom line  from bottom left to bottom right
   LJ_RAF( cBotRow, cLeftCol,  cBotWidth,  cThick, ;
            BOX_RAF_option,  BOX_line_pattern )

   do case

   case BOX_shadow == SHADOW_NE

        cSTopRow   :=  str( val( cTopRow   ) - 25, 10, 0 )
        cSLeftCol  :=  str( val( cLeftCol  ) + val(cThick) + 15 , 10, 0 )
        cSRightCol :=  str( val( cRightCol ) + val(cThick), 10, 0 )

        // draw top line first from top left to top right
        LJ_RAF( cSTopRow, cSLeftCol,  cWidth,  "25", ;
                HP_SHADING_PATTERN, SHADE_55_PERCENT )

        // Right side line from right top of box to right bottom
        LJ_RAF( cSTopRow, cSRightCol, "15", cHeight, ;
                HP_SHADING_PATTERN, SHADE_55_PERCENT )

   case BOX_shadow == SHADOW_SE

        cSTopRow   :=  str( val( cTopRow   ) + 25, 10, 0 )
        cSBotRow   :=  str( val( cBotRow   ) + val(cThick), 10, 0 )
        cSLeftCol  :=  str( val( cLeftCol  ) + val(cThick) + 15 , 10, 0 )
        cSRightCol :=  str( val( cRightCol ) + val(cThick), 10, 0 )

        // draw bottom shadow first from bottom left to bottom right
        LJ_RAF( cSBotRow, cSLeftCol,  cWidth,  "25", ;
                HP_SHADING_PATTERN, SHADE_55_PERCENT )


        // Right side line from right top of box to right bottom
        LJ_RAF( cSTopRow, cSRightCol, "15", cHeight, ;
                HP_SHADING_PATTERN, SHADE_55_PERCENT )


   case BOX_shadow == SHADOW_NW

        cSTopRow   :=  str( val( cTopRow ) - 25, 10, 0 )
        cSLeftCol  :=  str( val( cLeftCol ) - 15 , 10, 0 )

        // draw top line first from top left to top right
        LJ_RAF( cSTopRow, cSLeftCol,  cWidth,  "25", ;
                HP_SHADING_PATTERN, SHADE_55_PERCENT )

        // left side line from left top of box to left bottom
        LJ_RAF( cSTopRow, cSLeftCol,  "15", cHeight, ;
                HP_SHADING_PATTERN, SHADE_55_PERCENT )


   case BOX_shadow == SHADOW_SW

        cSTopRow   :=  str( val( cTopRow   ) + 25, 10, 0 )
        cSLeftCol  :=  str( val( cLeftCol  ) - 15 , 10, 0 )
        cSBotRow   :=  str( val( cBotRow   ) + val(cThick), 10, 0 )
        cSRightCol :=  str( val( cRightCol ) + val(cThick), 10, 0 )

        // draw bottom shadow first from bottom left to bottom right
        LJ_RAF( cSBotRow, cSLeftCol,  cWidth,  "25", ;
                HP_SHADING_PATTERN, SHADE_55_PERCENT )

        // Right side line from right top of box to right bottom
        LJ_RAF( cSTopRow, cSLeftCol, "15", cHeight, ;
                HP_SHADING_PATTERN, SHADE_55_PERCENT )


   endcase

return NIL

/*
   Function Ŀ
           Name: LJ_Line()                                                  
               :                                                            
    Description: define a line and print it                                 
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:13:24pm            Time updated: 07:13:24pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nXYfmt   = describes how to evaluate incomming parametest  
               :            IN = Inches   DT = Dots    RC = Row / Column    
               : nY       = top row (Y) of box                              
               : nX       = left most column (X) of the box                 
               : nW       = width of line (X)                               
               : nH       = height of line (Y)                              
               : nRAF     = numeric Fill Rectangular Area command           
               : nPattern = pattern for line density                        
   Return Value: nil                                                        
  
*/
function LJ_Line( nXYfmt, nY, nX, nW, nH, nRAF, nPattern)

   local cRow, cCol, cWidth, cHeight

   if nXYfmt == ROWCOL   //  parms passed in Row/Col config
      cRow    := lj_dots( nY, ROW_POSITION )
      cCol    := lj_dots( nX, COL_POSITION )
      cWidth  := lj_dots( nw, COL_POSITION )
      cHeight := lj_dots( nH, ROW_POSITION )

   else   //  parameters were passed in INCH, MILLIMETER or DOT config
      cRow    := lj_dots( nY, nXYfmt )
      cCol    := lj_dots( nX, nXYfmt )
      cWidth  := lj_dots( nw, nXYfmt )
      cHeight := lj_dots( nH, nXYfmt )
   endif

   // draw line from row,col for width x height
   LJ_RAF( cRow, cCol,  cWidth,  cHeight, nRAF,  nPattern )

return nil


/*
   Function Ŀ
           Name: LJ_RAF()                                                   
               :                                                            
    Description: define and fill a rectangle with either a                  
               : predefined pattern or with 1 of 16 shades of grey          
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:13:13pm            Time updated: 07:13:13pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: cYt      = top row (Y) of box                              
               : cXl      = left most column (X) of the box                 
               : cW       = width of box (X)                                
               : cH       = height of box (Y)                               
               : nRAF     = numeric Fill Rectangular Area command           
               : nPattern = pattern for line density                        
   Return Value: NIL                                                        
  
*/
static function LJ_RAF( cYt, cXl, cW, cH, nRAF, nPattern)
   local cs

   cs := ESC + "*p" + cXl + "x" + cYt + "Y" +;
         ESC + "*c" + cW  + "a" + cH  + "b"

   do case
      case nRAF == BLACK_FILL
           cs := cs + str(nRAF,1,0) + "P"

      case nRAF == WHITE_FILL
           cs := cs + str(nRAF,1,0) + "P"

      case nRAF == SHADED_FILL
           cs := cs + ltrim( str(nPattern,3,0) ) + "g" + str(nRAF,1,0) + "P"

      case nRAF == CROSSHATCH_FILL
           cs := cs + str(npattern,1,0) + "g" + str(nRAF,1,0) + "P"

      case nRAF == CURRENT_PATTERN_FILL
           cs := cs +  str(nRAF,1,0) + "P"
   endcase

   LJ_Send_CC( cS )

return  NIL

/*
   Function Ŀ
           Name: ROUNDIT()                                                  
               :                                                            
    Description: this fixes a bug in clipper/INTEL processor                
               :                                                            
               :                                                            
  Ĵ
   Date created: 02-27-96              Date updated: 02-27-96              
   Time created: 07:12:09pm            Time updated: 07:12:09pm            
      Copyright: (c) 1996 by Capitol City Software, Inc. All Rights Reserved.
  Ĵ
      Arguments: nNumber = the number to be rounded                         
               : nPlaces = the number of places to round to                 
   Return Value: nResult = the rounded number                               
  
*/
STATIC FUNCTION ROUNDIT(nNumber, nPlaces)
   local nResult := 0

   nPlaces := if(nPlaces == NIL, 0, nPlaces)

   nResult := if(nNumber < 0.0, -1.0, 1.0 ) * ;
              int( abs(nNumber) * 10^nPlaces + 0.50 + 10^-12) / 10 ^ nPlaces
RETURN nResult

